from base import AbstractGateway, GatewayRequest, GatewayResponse
import urllib

from quix.pay.transaction import CreditCard

class AimGateway(AbstractGateway):
    """ 
    Interface to the Authorize.Net Advanced Integration Method (AIM) payment
    gateway.
    
    The authentication login and password corresponds to the AIM API Login ID 
    and Transaction Key respectively. For example:
    
        gateway = AimGateway('<Your API Login ID>', '<Your Transaction Key>')
    
    This API is based on the "Advanced Integration Method (AIM) Developer Guide"
    version 2.0 published in June, 2010. Full documentation and examples
    are available at http:://www.quixotix.com/projects/quix.pay/
    
    Supported:
        - Authorization and Capture transactions using the sale() method.
        - Authorization Only transactions using the authorize() method.
        - Prior Authorization and Capture transactions using the capture() method.
        - Credit transactions using credit() method.
        - Void transactions using the void() method.
        - Customer details
        - Itemized order details
        
    Not Supported:
        - Capture Only transactions
        - Unlinked Credit transactions
        - Partial Authorization transactions
        - eCheck (use the eCheckGateway)
        - Automatic Recurring Payments (use the ArbGateway)
    
    """
    test_url = 'https://test.authorize.net/gateway/transact.dll'
    live_url = 'https://secure.authorize.net/gateway/transact.dll'

    API_VERSION = '3.1'
    DELIM_CHAR = ','
    
    def authorize(self, amount, card, order=None, customer=None):
        """ 
        Send a credit card AUTH_ONLY request to the Authorize.Net gateway.
        
        Args
            amount - The amount of the order.
            card - A CreditCard object containing valid credit card info.
            order - An optional Order object specifying order details.
            customer - An optional Customer object specifying customer info.

        Returns:
            A GatewayResponse object.
        """
        fields = self._create_common_fields(card, order, customer)
        fields.update({
            'x_type': 'AUTH_ONLY',
            'x_amount': str(amount),
        })

        # send request
        response = self.request(self.get_request_url(), 
                                self._prepare_fields(fields))

        return self.parse_response(response)
    
    def capture(self, trans_id):
        """ 
        Send a credit card PRIOR_AUTH_CAPTURE request to the Authorize.Net 
        gateway.
        
        Args
            trans_id - The transaction ID found in the GatewayResponse object
                returned from a previous call to the authorize() method.

        Returns:
            A GatewayResponse object.
        """
        fields = self._create_common_fields()
        fields.update({
            'x_type': 'PRIOR_AUTH_CAPTURE',
            'x_trans_id': trans_id,
        })

        # send request
        response = self.request(self.get_request_url(), 
                                self._prepare_fields(fields))

        return self.parse_response(response)

    def _create_address_fields(self, a, prefix=''):
        """ Creates a dictionary of the address fields """
        fields = {}
        
        if a.first_name is not None:
            fields['x_'+prefix+'first_name'] = a.first_name
        if a.last_name is not None:
            fields['x_'+prefix+'last_name'] = a.last_name
        if a.company is not None:
            fields['x_'+prefix+'company'] = a.company
        if a.address1 is not None or a.address2 is not None:
            fields['x_'+prefix+'address'] = "%s, %s" % (a.address1 or '', 
                                                        a.address2 or '')
        if a.city is not None:
            fields['x_'+prefix+'city'] = a.city
        if a.state_province is not None:
            fields['x_'+prefix+'state'] = a.state_province
        if a.postal_code is not None:
            fields['x_'+prefix+'zip'] = a.postal_code
        if a.country is not None:
            fields['x_'+prefix+'country'] = a.country
        
        return fields
        
    def _create_common_fields(self, card=None, order=None, customer=None):
        """ Creates a dictionary of fields common in most requests. """
        fields = {
            'x_login': self.login,
            'x_tran_key': self.password,
            'x_version': self.API_VERSION,
            'x_relay_response': 'FALSE',    # not used with AIM
            'x_delim_data': 'TRUE',         # "best practices" in all requests
            'x_delim_char': self.DELIM_CHAR
        }
        
        if self.use_test_mode:
            fields['x_test_request'] = 'TRUE'
        
        if card is not None:
            fields.update({
                'x_card_num': card.number,
                'x_exp_date': "%s%s" % (card.month, card.year) #MMYYYY
            })
            if card.code is not None:
                fields['x_card_code'] = card.code

        if customer is not None:
            if customer.ip is not None:
                fields['x_customer_ip'] = customer.ip
            if customer.email is not None:
                fields['x_email'] = customer.email
            if customer.cust_id is not None:
                fields['x_cust_id'] = customer.cust_id
            if customer.billing_address is not None:
                b = customer.billing_address
                fields.update(self._create_address_fields(b))
                if b.phone is not None:
                    fields['x_phone'] = b.phone 
                if b.fax is not None:
                    fields['x_fax'] = b.fax     
            if customer.shipping_address is not None:
                s = customer.shipping_address
                fields.update(self._create_address_fields(s, 'ship_to_'))
        
        if order is not None:
            # limit 30 items
            if order.order_id is not None:
                fields['x_invoice_num'] = order.order_id
            if order.description is not None:
                fields['x_description'] = order.description
            if order.items is not None:
                for item in order.items:
                    fields['x_line_item'] = "%s<|>%s<|>%s<|>%s<|>%s<|>" % (
                                                item.item_id or '',
                                                item.name or '',
                                                item.description or '',
                                                item.price or '',
                                                item.qty or ''
                                            )
        
        return fields
    
    def credit(self, trans_id, amount, card, order=None, customer=None):
        """ 
        Send a credit card CREDIT request to the Authorize.Net gateway.
        
        Args
            trans_id - The transaction ID found in the GatewayResponse object
                returned from a previous call to the authorize() or sale() 
                methods.
            amount - The amount to refund. Should be less than or equal to the 
                amount of the original transaction.
            card - A CreditCard object containing the original credit card 
                number or at least the last 4 digits of the credit card number.

        Returns:
            A GatewayResponse object.
        """
        fields = self._create_common_fields(card, order, customer)
        fields.update({
            'x_type': 'CREDIT',
            'x_trans_id': trans_id,
            'x_amount': str(amount)
        })

        # send request
        response = self.request(self.get_request_url(), 
                                self._prepare_fields(fields))

        return self.parse_response(response)
        
    def get_request_url(self):
        """
        Get the gateway request URL.
        
        Returns
            The Authorize.Net test gateway URL if in test mode, the production
            gateway URL otherwise.
        """
        if self.use_test_url:
            return self.test_url
        else:
            return self.live_url
         
    def parse_response(self, response):
        """ 
        Parses the response body into properties for convenience and abstraction
        in the client code. This should be called internally on every created
        GatewayResponse instance.
        
        response.status is set based on the "Response Code".
        response.message is set to the "Response Reason Text"
        response.trans_id is set to the "Transaction ID"
        
        The entire fields returned from Authorize.NET are stored as a dictionary
        in the response.gateway_data property. The keys correspond to the
        "field name" as defined in version 2.0 the AIM developer guide at 
        http://developer.authorize.net
        
        Args
            response - A GatewayResponse object
        
        Returns
            The udpated GatewayResponse object
        """
        response_offsets =  (
            'Response Code',
            'Response Subcode',
            'Response Reason Code',
            'Response Reason Text',
            'Authorization Code',
            'AVS Response',
            'Transaction ID',
            'Invoice Number',
            'Description',
            'Amount',
            'Method',
            'Transaction Type',
            'Customer ID',
            'First Name',
            'Last Name',
            'Company',
            'Address',
            'City',
            'State',
            'ZIP Code',
            'Country',
            'Phone',
            'Fax',
            'Email Address',
            'Ship To First Name',
            'Ship To Last Name',
            'Ship To Company',
            'Ship To Address',
            'Ship To City',
            'Ship To State',
            'Ship To ZIP Code',
            'Ship To Country',
            'Tax',
            'Duty',
            'Freight',
            'Tax Exempt',
            'Purchase Order Number',
            'MD5 Hash',
            'Card Code Response',
            'Cardholder Authentication Verification Response',
            'Account Number',
            'Card Type',
            'Split Tender ID',
            'Requested Amount',
            'Balance On Card'
        )
        
        fields = response.body.split(self.DELIM_CHAR)    
        response.gateway_data = {}
        
        # TODO: comprehension?
        for i, field in enumerate(fields):
            if i >= len(response_offsets):
                break;
            response.gateway_data[response_offsets[i]] = field
        
        if fields[0] == '1':
            response.status = GatewayResponse.APPROVED
        elif fields[0] == '2':
            response.status = GatewayResponse.DECLINED
        elif fields[0] == '3':
            response.status = GatewayResponse.ERROR
        else:
            response.status = GatewayResponse.UNKNOWN
        
        response.message = fields[3]
        response.avs_result = fields[5]
        response.trans_id = fields[6]
        
        return response
    
    def _prepare_fields(self, fields):
        """ Prepare fields for sending to Authorize.net """
        # a comma would break the CSV response
        fields = dict([k,str(v).replace(',', '')] for (k,v) in fields.iteritems())
        fields['x_delim_char'] = self.DELIM_CHAR # in-case it's a comma
        
        return urllib.urlencode(fields)
    
    def sale(self, amount, card, order=None, customer=None):
        """ 
        Send a credit card AUTH_CAPTURE request to the Authorize.Net gateway.
        
        Args
            amount - The amount of the order.
            card - A CreditCard object containing valid credit card info.
            order - An optional Order object specifying order details.
            customer - An optional Customer object specifying customer info.

        Returns:
            A GatewayResponse object.
        """
        fields = self._create_common_fields(card, order, customer)
        fields.update({
            'x_type': 'AUTH_CAPTURE',
            'x_amount': str(amount),
        })

        # send request
        response = self.request(self.get_request_url(), 
                                self._prepare_fields(fields))

        return self.parse_response(response)
        
    def __str__(self):
        return "Authorize.Net Advanced Integration Method (AIM)"
    
    def test_response(self, response_code):
        """
        Test a response code.
        
        The Authorize.Net API provides a means of testing each response reason
        code using a special credit card number and specifying the response
        code you want the server to return as the dollar amount of the sale. 
        This method does just that.
        
        Args
            response_code - The response code you want to test
        
        Returns
            The GatewayResponse object from a call to the sale() method.
        """
        mode = self.test_mode
        self.test_mode = True
        card = CreditCard(
            number = '4222222222222',
            month = '10',
            year = '2020',
            first_name = 'John',
            last_name = 'Doe',
            code = '123'    
        )
        response = self.sale(card, response_code)
        self.test_mode = mode
        return response
    
    def void(self, trans_id, card=None, order=None, customer=None):
        """ 
        Send a credit card VOID request to the Authorize.Net gateway.
        
        Args
            trans_id - The transaction ID found in the GatewayResponse object
                returned from a previous call to the authorize() or sale() 
                methods.

        Returns:
            A GatewayResponse object.
        """
        fields = self._create_common_fields(card, order, customer)
        fields.update({
            'x_type': 'VOID',
            'x_trans_id': trans_id
        })

        # send request
        response = self.request(self.get_request_url(), 
                                self._prepare_fields(fields))

        return self.parse_response(response)



from base import AbstractGateway
from authorizenet import AimGateway
from quix.pay.transaction import CreditCard

class AimEmulationGateway(AimGateway):
    """ 
    Interface to the QuantamGateway using Authorize.Net Emulation.
    
    The QuantamGateway provides a basic emulation mode of the Authorize.Net
    payment gateway. This class simply extends the AimGatewy class to utilize
    the QuantamGateway URL instead. Since this is only a basic emulation, you
    should refer to the QuantamGateway documentation at    
    https://www.quantumgateway.com/view_developer.php?Cat1=6 to understand
    it's limitations.
    """
    live_url = 'https://secure.quantumgateway.com/cgi/authnet_aim.php'
    test_url = live_url
        
    def __str__(self):
        return "QuantamGateway Authorize.Net AIM Emulation"

class TransparentDbEngine(AbstractGateway):
    pass




from exceptions import NotImplementedError
import urllib2

from quix.pay.transaction import CreditCard

class AbstractGateway(object):
    """
    Abstract class for interfacing with payment gateways. 
    """
    test_mode = False
    request_timeout = 30
    use_test_url = False
    use_test_mode = False
    
    @property
    def live_url(self):
        """ The gateway API URL used for test requests. """
        raise NotImplementedError("Property not implemented in gateway class: 'live_url'")
        
    @property
    def test_url(self):
        """ The gateway API URL used for production requests. """
        raise NotImplementedError("Property not implemented in gateway class: 'test_url'.")

    def authentication(self, login, password):
        """
        Specify payment gateway authentication.
        
        Args
            login - A string used by the gateway to identify a user of the API.
            password - A string used by the gateway to authenticate the user.
        """
        self.login = login
        self.password = password
        
    def authorize(self, amount, card, order=None, customer=None):
        """ 
        Send a credit card authorization request to the gateway.
        
        Args
            amount - The amount of the order.
            card - A CreditCard object containing valid credit card info.

        Returns:
            A GatewayResponse object.
        """
        raise NotImplementedError("Method not implemented in gateway class: 'authorize'.")

    def capture(self, trans_id):
        """ 
        Send a credit card capture request to the gateway.
        
        Args
            card - A CreditCard object containing valid credit card info.
            trans_id - The transaction ID found in the GatewayResponse object
                returned from a previous call to the authorize() method.

        Returns:
            A GatewayResponse object.
        """
        raise NotImplementedError("Method not implemented in gateway class: 'capture'.")
    
    def credit(self, trans_id, amount, card=None, order=None, customer=None):
        """ 
        Send a credit card refund request to the gateway.
        
        Args
            trans_id - The transaction ID found in the GatewayResponse object
                returned from a previous call to the authorize() or sale() 
                methods.
            amount - The amount to refund. Should be less than or equal to the 
                amount of the original transaction.
            card - A CreditCard number or the last 4 digits of a credit card
                number may be required by some gateways.

        Returns:
            A GatewayResponse object.
        """
        raise NotImplementedError("Method not implemented in gateway class: 'credit'.")
        
    def get_last_request(self):
        """
        Gets the GatewayRequest instance from the last call to the request()
        method.
        
        Returns
            An existing GatewayRequest instance.
        """
        return self._last_request
    
    def __init__(self, login=None, password=None):
        """
        Optionally provide arguments to the authentication() method as a 
        convenience.
        """
        self.authentication(login, password)
        self._last_request = None
        
    def request(self, url, data):
        """
        Submits an http(s) request to the provided gateway url. This response is
        saved internally and accessible using the get_last_request() method.
        
        Args
            url - The gateway url
            data - The data to post to the gateway url
            
        Returns
            A new GatewayResponse instance
        """
        self._last_request = GatewayRequest(url, data, self.request_timeout)
        response = self._last_request.send()
        
        return response
    
    def sale(self, amount, card, order=None, customer=None):
        """ 
        Send a credit card authorization and capture request to the gateway.
        
        Args
            amount - The amount of the order.
            card - A CreditCard object containing valid credit card info.
            order - An optional Order object specifying order details.
            customer - An optional Customer object specifying customer info.

        Returns
            A GatewayResponse object.
        """
        raise NotImplementedError("Method not implemented in gateway class: 'sale'.")
 
    def void(self, trans_id, card=None, order=None, customer=None):
        """ 
        Send a credit card void request to the gateway.
        
        Args
            trans_id - The transaction ID found in the GatewayResponse object
                returned from a previous call to the authorize() or sale() 
                methods.
            amount - The amount to refund. Should be less than or equal to the 
                amount of the original transaction.

        Returns:
            A GatewayResponse object.
        """
        raise NotImplementedError("Method not implemented in gateway class: 'void'.")
        
class GatewayRequest(object):
    """
    An http(s) request to the payment gateway.
    
    An instance of a GatewayRequest object provides url and POST data from the 
    request so that client applications may choose to further process or log
    the request details.
    """
    
    def __init__(self, url, data, timeout=30):
        """
        Initialize an http(s) request.
        
        Args
            url - The full URL for the request.
            data - The raw text data to post to the request url.
            timeout - Time in seconds to timeout the request.
        """
        self.url = url
        self.data = data
        self.timeout = timeout
    
    def send(self):
        """
        Send an http(s) request.
        
        Returns
            A new instance of a GatewayResponse object.
        """
        r = urllib2.urlopen(self.url, self.data, self.timeout)
        response = GatewayResponse(r.read(), r.info())
        return response
    
class GatewayResponse(object):
    """
    An http(s) response from the payment gateway.
    
    An instance of the GatewayResponse provides details of the response from 
    the payment gateway.
    """
    # constants for the status of the transaction
    status_strings = ('Unknown', 'Approved', 'Declined', 'Error')
    UNKNOWN = 0
    APPROVED = 1
    DECLINED = 2
    ERROR = 3
    
    # must be set by a gateway class instance    
    status = 0
    message = ''
    trans_id = None
    avs_result = None
    gateway_data = None
    
    def __init__(self, body, headers=None):
        """
        Initializes the GatewayResponse instance with the full contents of the 
        http response.
        
        Args
            body - The raw, un-parsed http body from the response.
            headers - The http headers from the response.
        """
        self.body = body
        self.headers = headers




class Address(object):
    """
    Address used in a payment gateway transaction.
    """
    def __init__(self, first_name=None, last_name=None, company=None, 
                 address1=None, address2=None, city=None, state_province=None,
                 postal_code=None, country=None, phone=None, fax=None):
        """
        Initializes a new mailing address. All arguments should be strings or 
        None.
        
        Args
            first_name - First name
            last_name - Last name
            company - A company/organization name or None.
            address1 - First line of the street address.
            address2 - Second line of street address (such as APT or SUITE) or
                None.
            city - Name of the city.
            state_province - State or province in ISO 3166-2 or FIPS 10-4 
                string e.g., 'CA' for California, 'ON' for Ontario, etc.
                http://www.iso.org/iso/country_codes/background_on_iso_3166/iso_3166-2.htm
                http://www.itl.nist.gov/fipspubs/fip10-4.htm
            postal_code - 5-digit US Zip Code or a postal code as a string e.g., 
                'L5NV2', '97217', etc.
            country - ISO 3166 Country Code as a string e.g., 'US', 'GB', etc.
                http://www.iso.org/iso/english_country_names_and_code_elements
            phone - A phone number as a string e.g., '(555) 555-5555'.
            fax - A fax number as a string. e.g., '(555) 555-5555'.

        """
        self.first_name = first_name
        self.last_name = last_name
        self.company = company
        self.address1 = address1
        self.address2 = address2
        self.city = city
        self.state_province = state_province.upper()
        self.postal_code = postal_code
        self.country = country.upper()
        self.phone = phone 
        self.fax = fax 
        
    def __str__(self):
        sections = [
            "%s %s" % (self.first_name or '', self.last_name or ''),
            self.company or '',
            self.address1 or '',
            self.address2 or '',
            self.city or '',
            "%s %s %s" % (self.state_province or '', self.postal_code or '', 
                          self.country or '')
        ]
        return ', '.join(sections)
    
class Check(object):
    """
    Electronic check information used in a payment gateway transaction.
    """
    pass
    
class CreditCard(object):
    """
    Credit card information used in a payment gateway transaction.
    """
    TYPES = (
        'Visa',
        'MasterCard',
        'American Express',
        'Discover',
        'Diners Club',
        'JCB'
    )
    
    def __init__(self, number, first_name, last_name, month, year, code=None):
        self.number = number
        self.first_name = first_name
        self.last_name = last_name
        self.month = month
        self.year = year
        self.code = code
        self.type = None
    
    def is_valid(self):
        """ 
        Checks if the credit card number is valid.
        """

        return True

class Customer(object):
    """
    Customer information used in a payment gateway transaction.
    """
    def __init__(self, cust_id=None, email=None, ip=None, billing_address=None, 
                 shipping_address=None):
        """
        Initializes a Customer instance.
        
        Args
            cust_id - A merchant assigned ID for the customer
            email - A valid email address string or None.
            ip - An IP address e.g. '255.255.255.255' or None.
            billing_address - An Address object instance or None.
            shipping_address - An Address object instance or None.
        """
        self.cust_id = cust_id
        self.email = email
        self.ip = ip
        self.billing_address = billing_address
        self.shipping_address = shipping_address

class Item(object):
    """
    Item information used in an Order passed to a payment gateway transaction.
    """
    def __init__(self, item_id=None, name=None, description=None, qty=None, 
                 price=None):
        """
        Initializes an Item instance.
        
        Args
            item_id - The ID of SKU of the item or None
            name - The name of the item or None
            description - A short description of the item or None
            qty - The quantity of the item purchased or None
            price - The price per unit of the item or None
        """
        self.item_id = item_id
        self.name = name
        self.description = description
        self.qty = qty
        self.price = price
        
    
class Order(object):
    """
    Order information used in a payment gateway transaction.
    """
    def __init__(self, order_id=None, description=None, items=None):
        """
        Initializes an Order instance.
        
        Args
            order_id - The unique order ID or None
            description - A short description of the order or None
            items - A list of Item objects or None
        """
        self.order_id = order_id
        self.description = description
        self.items = items

       



from setuptools import setup, find_packages
import os

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='quix.pay', 
    version='0.1.0',
    description='Abstract interfacing with online payment gateways.',
    long_description=read('README'),
    author='Micah Carrick, Don Morris',
    url='http://www.quixotix.com',
    author_email='software@quixotix.com',
    packages=find_packages(),
    namespace_packages=['quix'],
    install_requires=['setuptools'],
    license='BSD',
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Office/Business :: Financial",
        "Topic :: Software Development :: Libraries ",
    ],
    keywords = 'payment gateway processor credit card authorize.net AIM paypal'
)



import base
from xml.etree import ElementTree

class XmlMessengerGateway(base.Gateway):
    test_url = 'https://dev.psigate.com:7989/Messenger/XMLMessenger'
    live_url = 'https://secure.psigate.com:7934/Messenger/XMLMessenger'

    def authorize(self, card, amount, order_num=None, description=None):
        """ Submit a card authorization request. """
        xml = self._head_xml()
        xml += "\t<CardAction>1</CardAction>\n"
        xml += "\t<Subtotal>%s</Subtotal>\n" % str(amount)
        xml += self._card_xml(card)
        
        if order_num:
            xml += "\t<OrderID>%s</OrderID>\n" % order_num
       
        if description:
            xml += "\t<Comments>%s</Comments>\n" % description
            
        if self.test_mode:
            post_url = self.test_url
        else:
            post_url = self.live_url
        
        xml += self._foot_xml()
        
        request = gateway.Request(post_url, xml, self.request_timeout)
        response = request.send()
        self.parse_response(response)
        
        return (request, response)
    
    def _card_xml(self, card):
        xml = "<CardNumber>%s</CardNumber>" % card.number
        xml += "<CardExpMonth>%s</CardExpMonth>" % card.month
        xml += "<CardExpYear>%s</CardExpYear>" % card.year[2:]
        
        return xml
        
    def _foot_xml(self):
        xml = "</Order>";
        return xml
        
    def _head_xml(self):
        xml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
        xml += "<Order>"
        xml += "<StoreId>%s</StoreId>" % self.login
        xml += "<Passphrase>%s</Passphrase>" % self.password
        xml += "<PaymentType>CC</PaymentType>"
        
        return xml
    
    def parse_response(self, response):
        element = ElementTree.XML(response.body)
        approved = element.find('Approved')
        errmsg = element.find('ErrMsg')
       
        if approved is None or errmsg is None:
            response.status = gateway.Response.UNKNOWN
            response.status_text = "Failed parsing response XML."
            return response

        response.status_text = approved.text
        
        if response.status_text == 'APPROVED':
            response.status = gateway.Response.APPROVED
        elif response.status_text == 'DECLINED':
            response.status = gateway.Response.DECLINED
        elif response.status_text == 'ERROR':
            response.status = gateway.Response.ERROR
            response.status_text = errmsg.text
        else:
            approved.status = gateway.Response.UNKNOWN
        
        trans = element.find('TransRefNumber')
        if trans is not None:
            response.trans_id = trans.text

        return response
        
    def __str__(self):
        return "PSiGate XML Messenger Interface"
